@@ -1,5 +1,5 @@
 #!/bin/sh
-# regression-test-0.3.7.sh - check consistency of mpgrafic behaviour versus v0.3.7
+# regression-test-0.3.7.9.sh - check consistency of mpgrafic behaviour versus v0.3.7.9
 
 #/***************************************************************************
 # *   Copyright (C) 2017 Boud Roukema                                       *
@@ -23,7 +23,7 @@
 
 
 printf "Code regression test: does the standard input file generate an\n"
-printf "output file identical to that produced by version 0.3.7\n"
+printf "output file identical to that produced by version 0.3.7.9\n"
 printf "of mpgrafic? IEEE_UNDERFLOW_FLAG and IEEE_DENORMAL exceptions\n"
 printf "are considered acceptable and ignored. Other minor warnings are\n"
 printf "ignored too on some systems/implementations.\n\n"
@@ -52,6 +52,15 @@ TMP_OUT_CLEANED=Output.stdout.np32.tmp.2
 ##
 ## Please add cases for other mpi implementations or GNU/Linux
 ## distribution systems, or propose a more generic solution.
+##
+## 2017-08-14: Switch --mca plm_rsh_agent ssh to
+## --mca plm_rsh_agent /bin/false - see
+##  https://lists.debian.org/debian-science/2015/10/msg00032.html +
+##  http://www.open-mpi.de/faq/?category=rsh
+## This may enable builds on kfreebsd-(amd64|i386) and does not
+## reduce the power of the test: ssh is obviously
+## *not* desirable in automated builds of mpgrafic, and is not tested.
+
 ##
 if [ -r /usr/share/mpi-default-dev/debian_defaults ] ; then
     MPI_IMPL=$(grep ARCH_DEFAULT_MPI_IMPL \
@@ -60,18 +69,19 @@ if [ -r /usr/share/mpi-default-dev/debian_defaults ] ; then
                       tail -n 1) # last evaluation overrides
     if [ "x${MPI_IMPL}" = "xopenmpi" ]; then
         printf "This looks like a debian openmpi system.\n"
-        mpirun -n 1 --mca plm_rsh_agent sh \
-               ${srcdir}/src/mpgrafic --np=32 < ${srcdir}/Input.stdin \
+        mpirun -n 1 --mca plm_rsh_agent /bin/false \
+               ${MPI_ALLOW_RUN_AS_ROOT} \
+               ${srcdir}/src/mpgrafic --np=32 ${srcdir}/Input.stdin \
                > ${TMP_OUT}
     else
-        printf "This looks like a debian system, but \n"
+        printf "Warning: This looks like a debian system, but \n"
         printf "options for ${MPI_IMPL} have not yet been implemented,\n"
         printf "so this test will pass without doing any work.\n"
         exit 0
     fi
 else
     printf "MPI implementation unknown.\n"
-    mpirun -n 1 ${srcdir}/src/mpgrafic --np=32 < ${srcdir}/Input.stdin > ${TMP_OUT}
+    mpirun -n 1 ${srcdir}/src/mpgrafic --np=32 ${srcdir}/Input.stdin > ${TMP_OUT}
 fi
 
 ## Ignore known warnings from non-openmpi implementations of mpi:
@@ -79,12 +89,17 @@ cat ${TMP_OUT} | \
     grep -v "Warning: Cannot receive number of cores .*Round robin.*startup scheme will be involved" > \
          ${TMP_OUT_CLEANED}
 
-LINES_DIFF=$(diff ${EXPECTED_OUTPUT} ${TMP_OUT_CLEANED} | wc -l)
 
-if [ "x${LINES_DIFF}" = "x0" ] ; then
-    printf "A 32^3 mpi run of mpgrafic matched the expected output.\n"
+if ( [ -r ${EXPECTED_OUTPUT} ] && [ -r ${TMP_OUT_CLEANED} ] ); then
+    LINES_DIFF=$(diff ${EXPECTED_OUTPUT} ${TMP_OUT_CLEANED} | wc -l)
+    if [ "x${LINES_DIFF}" = "x0" ] ; then
+        printf "A 32^3 mpi run of mpgrafic matched the expected output.\n"
+    else
+        printf "Warning: A 32^3 mpi run of mpgrafic did not match the expected output.\n"
+        exit 1
+    fi
 else
-    printf "Warning: A 32^3 mpi run of mpgrafic did not match the expected output.\n"
+    printf "Warning: ${EXPECTED_OUTPUT} or ${TMP_OUT_CLEANED} is not found or not readable.\n"
     exit 1
 fi
 #rm -f ${TMP_OUT}
