@@ -1,6 +1,6 @@
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
 !   mpgrafic - MPI version of N-body initial conditions grafic package   !
-!   Copyright (C) 2008-2017 by Simon Prunet, Christophe Pichon et al     !
+!   Copyright (C) 2008-2019 by Simon Prunet, Christophe Pichon et al     !
 !   prunet iap.fr                                                        !
 !                                                                        !
 !   This program is free software; you can redistribute it and/or modify !
@@ -100,6 +100,17 @@ program grafic1
   double precision dsigmadispl
   parameter (twopi=6.283185307179586d0)
   logical hanning
+
+  ! The input parameter file can be named in the command line or piped from standard input.
+#ifdef f2003
+  use, intrinsic :: iso_fortran_env, only : input_unit
+#else
+#  define input_unit 5
+#endif
+  character(len=256) :: input_param_file
+  integer :: input_param_file_id = 15
+  integer :: infile_id = input_unit   ! By default, read from standard input.
+
   character(len=128) :: filename, small_file_name, small_kfile_name
   real astart,omegam,omegav,h0,dladt,fomega,vfact,pbar,pcdm
   real omegab,omegahdm,degen_hdm
@@ -114,7 +125,8 @@ program grafic1
   type(cosmo) :: headc, shcosmo
   character(len=128) :: outputfile
 
-  character(len=128) :: command_line_arg1 ! so far very limited options
+  character(len=256) :: command_line_arg1 ! so far very limited options
+  character(len=256) :: command_line_arg2 ! so far very limited options
   logical :: help_only_arg = .false.
 
   common /cosmoparms/ omegam,omegav,h0
@@ -132,7 +144,7 @@ program grafic1
         write(*,'(a)')PACKAGE_STRING
         write(*,'(a)')'Written by Simon Prunet, Christophe Pichon et al.'
         write(*,'(a)')''
-        write(*,'(a)')'Copyright (C) 2008-2017 Prunet, Pichon, Roukema.'
+        write(*,'(a)')'Copyright (C) 2008-2019 Prunet, Pichon, Roukema.'
         write(*,'(a)')'This program is free software; you may redistribute it under the terms of'
         write(*,'(a)')'the GNU General Public License version 2 or later.  This program has'
         write(*,'(a)')'absolutely no warranty.'
@@ -141,7 +153,7 @@ program grafic1
           command_line_arg1 == '--help')then
         help_only_arg = .true.
         ! TODO: modularise this out to a documentation type file
-        write(*,'(a)')'Usage: mpgrafic [OPTION...] [< INPUT_FILE] [> OUTPUT_FILE]'
+        write(*,'(a)')'Usage: mpgrafic [OPTION...] [[<] INPUT_FILE] [> OUTPUT_FILE]'
         write(*,'(a)')''
         write(*,'(a)')'mpgrafic produces initial conditions files for N-body simulations of'
         write(*,'(a)')'cosmological large-scale structure formation. It is a free-licensed,'
@@ -163,7 +175,7 @@ program grafic1
      elseif(command_line_arg1 == '--usage')then
         help_only_arg = .true.
         write(*,'(a)')'Usage: mpgrafic [-?hV] [--np=INTEGER] [--help] [--usage]'
-        write(*,'(a)')'            [--version] [< INPUT_FILE] [> OUTPUT_FILE]'
+        write(*,'(a)')'            [--version] [[<] INPUT_FILE] [> OUTPUT_FILE]'
      elseif( command_line_arg1(1:5) == '--np=' .and. &
           len(command_line_arg1) .gt. 5 )then
         read(command_line_arg1(6:len(command_line_arg1)), &
@@ -174,12 +186,29 @@ program grafic1
         else
            write(*,'(a)')'Warning: error reading option --np.'
         endif
+     elseif( .not.(command_line_arg1(1:1) == '-') .and. &
+          len(command_line_arg1) .gt. 0 )then
+        ! assume that this is a filename
+        input_param_file = trim(command_line_arg1)
+        infile_id = input_param_file_id
      endif
      if(help_only_arg)then ! if valid command line argument found
         stop ! user only wants to know how to get started
      endif
   endif one_command_line_argument
 
+  second_command_line_argument: if(command_argument_count() .gt. 1)then
+     call get_command_argument(2,command_line_arg2)
+     command_line_arg2 = trim(command_line_arg2)
+     if( .not. (infile_id == input_param_file_id) .and. & ! if filename not yet found
+          .not.(command_line_arg2(1:1) == '-') .and. & ! refuse filenames starting with '-'
+          len(command_line_arg2) .gt. 0 )then ! refuse zero-length filenames
+        ! assume that this is a filename
+        input_param_file = trim(command_line_arg2)
+        infile_id = input_param_file_id
+     endif
+  endif second_command_line_argument
+
   !
   call mpi_init(ierr)
   call mpi_comm_rank(mpi_comm_world,myid,ierr)
@@ -196,15 +225,24 @@ program grafic1
   ny=np2
   nz=np3
 
+  if (myid==0) then
+     ! open file if stdin not used
+     if(infile_id == input_param_file_id)then
+        open(infile_id, file=input_param_file, access='sequential',&
+             status='old', form='formatted')
+
+     endif
+  endif
+
   !
   !  Initialize power spectrum.
-  call pini
+  call pini(infile_id)
   !  Initialize grid spacing.
   if (myid==0) then
      print*,'Enter dx (initial particle spacing in Mpc, not Mpc/h)'
      print*,'   or enter -boxlength in Mpc/h'
      print*,'   i.e. dx in Mpc if > 0, or -boxlength in Mpc/h if < 0'
-     read(*,*) dx
+     read(infile_id,*) dx
      write(*,'(f15.5)') dx
   endif
   call mpi_bcast(dx,1,mpi_real,0,mpi_comm_world,ierr)
@@ -220,7 +258,7 @@ program grafic1
      print*,'  enter the ultimate refinement factor for the ', &
           &    'smallest grid spacing'
      print*,'  (This is used only to set astart.)'
-     read(*,*) nrefine
+     read(infile_id,*) nrefine
      print*, nrefine
   endif
   call mpi_bcast(nrefine,1,mpi_integer,0,mpi_comm_world,ierr)
@@ -290,7 +328,7 @@ program grafic1
   !
   if (myid==0) then
      print*,'Enter 0 for final output or 1 for further refinement'
-     read(*,*) irand
+     read(infile_id,*) irand
      print*, irand
   endif
   call mpi_barrier(mpi_comm_world,ierr)
@@ -383,13 +421,13 @@ program grafic1
   if (myid==0) then
      print*,'Setting output grid to ',m1s,m2s,m3s
      print*,'Enter <RETURN> to skip output grid size'
-     read(*,*)
+     read(infile_id,*)
      print*,'Enter <RETURN> to skip output grid offset'
-     read(*,*)
+     read(infile_id,*)
      print*,'Enter <RETURN> to skip final grid size'
-     read(*,*)
+     read(infile_id,*)
      print*,'Enter <RETURN> to skip final grid offset'
-     read(*,*)
+     read(infile_id,*)
      print*,'Will produce output files ic_deltab, ic_velb[xyz],', &
           &           ' ic_velc[xyz]'
   endif
@@ -404,7 +442,7 @@ program grafic1
      print*,'    irand=1 to generate new noise and save to file'
      print*,'    irand=2 to read noise from existing file'
      print*,'Enter irand'
-     read(*,*) irand
+     read(infile_id,*) irand
      print*, irand
   endif
   call mpi_barrier(mpi_comm_world,ierr)
@@ -418,7 +456,7 @@ program grafic1
   if (myid==0) then
      print*,'  Enter random number seed (9-digit integer, ignored ', &
           &    'if irand=2)'
-     read(*,*) iseed
+     read(infile_id,*) iseed
      write(*,'(i12)') iseed
   endif
   call mpi_barrier(mpi_comm_world,ierr)
@@ -431,7 +469,7 @@ program grafic1
   if (myid==0) then
      print*,'  Enter filename of white noise file (or <RETURN> ', &
           &    'if irand=0)'
-     read(*,'(a)') filename
+     read(infile_id,'(a)') filename
      print*,filename
   endif
   call mpi_barrier(mpi_comm_world,ierr)
@@ -443,7 +481,7 @@ program grafic1
   ! Ask if padding of small k modes is required
   if (myid==0) then
      print*,' Enter 1 if padding of large scales is required, 0 otherwise'
-     read(*,*) ipad
+     read(infile_id,*) ipad
      print*, ipad
   endif
   call mpi_barrier(mpi_comm_world,ierr)
@@ -451,8 +489,13 @@ program grafic1
   small_file_name=''
   if (myid==0) then
      print*,' Enter padding file name (white noise format, ignored if ipad=0)'
-     read(*,'(a)') small_file_name
+     read(infile_id,'(a)') small_file_name
      print*, small_file_name
+
+     ! close file if stdin not used
+     if(infile_id == input_param_file_id)then
+        close(infile_id)
+     endif
   endif
   call mpi_barrier(mpi_comm_world,ierr)
   call mpi_bcast(small_file_name,128,mpi_character,0,mpi_comm_world,ierr)
