@@ -1,5 +1,5 @@
 !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-!   Copyright (C) 2008-2017 by Simon Prunet, Christophe Pichon et al     !
+!   Copyright (C) 2008-2019 by Simon Prunet, Christophe Pichon et al     !
 !   prunet iap.fr                                                        !
 !                                                                        !
 !   This program is free software; you can redistribute it and/or modify !
@@ -511,12 +511,16 @@ function pbar(ak,a)
   return
 
 end function pbar
+
+
 !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
-subroutine pini
+subroutine pini(infile_id)
 #ifdef HAVE_USE_MPI_F08
   use mpi_f08
 #endif
 
+  integer :: infile_id
+
   !  Pini initializes the power spectrum.
   !parameter (pi=3.1415926535898d0)
   double precision, parameter :: pi=3.1415926535898d0
@@ -565,14 +569,14 @@ subroutine pini
      write(*,*) '    3 for T(k)=1 (scale-free)' ! TODO: See uninitialised warning below and fix.
      write(*,*) '    4 for T(k) from approx. analytical fit (E&Hu)'
      write(*,*) 'Enter case (1,2,3 or 4) now:'
-     read(*,*) icase
+     read(infile_id,*) icase
      print*, icase
   endif
   call mpi_bcast(icase,1,mpi_integer,0,mpi_comm_world,ierr)
   if (icase.eq.1) then
      if (myid==0) then
         write(*,*) 'Enter linger.dat filename'
-        read(*,'(a)') filename
+        read(infile_id,'(a)') filename
         open(11,file=filename,status='old')
         rewind 11
         read(11,*) omegab,omegac,omegav,omegan
@@ -606,7 +610,7 @@ subroutine pini
         if (myid==0) then
            write(*,*) 'Enter Omega_m, Omega_v, H0 (km/s/Mpc,', &
                 &               ' set H0=1 for scale-free case)'
-           read(*,*) omegam,omegav,h0
+           read(infile_id,*) omegam,omegav,h0
            write(*,'(3f15.5)') omegam,omegav,h0
         endif
         call mpi_bcast(omegam,1,mpi_real,0,mpi_comm_world,ierr)
@@ -616,7 +620,7 @@ subroutine pini
         if(icase.eq.4) then
            if (myid==0) then
               write(*,*) 'Enter Omega_b'
-              read(*,*) omegab
+              read(infile_id,*) omegab
               write(*,'(f15.5)') omegab
            end if
            call mpi_bcast(omegab,1,mpi_real,0,mpi_comm_world,ierr)
@@ -641,7 +645,7 @@ subroutine pini
   if (myid==0) then
      write(*,*) 'Enter long-wave spectral index n (scale-invariant', &
           &             ' is n=1)'
-     read(*,*) an
+     read(infile_id,*) an
      write(*,'(f15.5)') an
   endif
   call mpi_bcast(an,1,mpi_integer,0,mpi_comm_world,ierr)
@@ -654,7 +658,7 @@ subroutine pini
      else
         write(*,*) '  k^3*P(k) at k=pi = ?'
      end if
-     read(*,*) anorml
+     read(infile_id,*) anorml
   endif
   call mpi_bcast(anorml,1,mpi_real,0,mpi_comm_world,ierr)
 
@@ -839,7 +843,7 @@ subroutine pini
           &             ' kmax (1/Mpc)'
      write(*,*) 'Enter 0,0 if you don''t want the power spectrum', &
           &             ' written to a file'
-     read(*,*) ak1,ak2
+     read(infile_id,*) ak1,ak2
      write(*,'(2es15.5)') ak1,ak2
   endif
 
